import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;
import java.io.*;
import java.util.Scanner;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.AlphaComposite;
import java.awt.image.BufferedImage;

import javax.imageio.ImageIO;
import java.io.*;

/**
 * Write a description of class ControlBox here.
 * 
 * @author Taylor Born
 * @version January 2011
 */
public class ControlBox  extends Actor
{
    // Images used for drawing
    private static GreenfootImage blockPic = new GreenfootImage("rubble.png");
    private static GreenfootImage stonePic = new GreenfootImage("stone.jpg");
    private static GreenfootImage fogPic = new GreenfootImage("foggy.png");
    
    // 2D representation of environment, used in determining presence of obstacles
    private char[][] grid;
    
    // Objects to interact in environment
    private ArrayList<Point> blocks;
    private ArrayList<Light> lights;
    private ArrayList<Shade> shades;
    private ArrayList<Missle> missles;
    private ArrayList<Burst> bursts;
    private Ship myShip;
    
    // Keep track of executions pacing the amount of times the screen is drawn and updated
    private int skip;
    
    private int score;
    private Font scoreFont = new Font("Times New Roman", Font.BOLD, 25);
    private int centerScore;
    private int centerHealth;
    private double darkness;
    private String keysPressed = "";
    
    private boolean mainScreen = true;
    
    private static final int CODE_ID = 45;

    public ControlBox()
    {
        Shade.cb = this;
        paint();
    }
    
    /*
     * Set up environment for game
     */
    public void newGame()
    {
        // Construct our new ship with given map location
        myShip = new Ship(new Point(50, 50));
        
        centerHealth = 144;
        centerScore = 13;
        darkness = 25;
        score = 0;
        
        try
        {
            // Read in from file
            BufferedReader br = new BufferedReader(new InputStreamReader(getClass().getClassLoader().getResource("Level.txt").openStream()));
            String str = br.readLine();
            
            // List of strings to be held till end for creating 2D representation
            ArrayList<String> strs = new ArrayList<String>();
            
            // Reset lists of objects that will be in the game
            missles = new ArrayList<Missle>();
            bursts = new ArrayList<Burst>();
            lights = new ArrayList<Light>();
            blocks = new ArrayList<Point>();
            shades = new ArrayList<Shade>();
            
            int next = 10 + Greenfoot.getRandomNumber(3);
            
            // Loop through each line, add a block if necessary
            while (!str.equals(""))
            {
                for (int i = 0; i < str.length(); i++)
                    if (str.charAt(i) == '#')
                        blocks.add(new Point(i * 30, strs.size() * 30));
                    else if ((strs.size() - 1) % 3 == 0)
                    {
                        if (--next == 0)
                        {
                            shades.add(new Shade(i * 30 + 5 + Greenfoot.getRandomNumber(20), strs.size() * 30 + 5 + Greenfoot.getRandomNumber(20)));
                            next = 2 + Greenfoot.getRandomNumber(9);
                        }
                    }
                strs.add(str);
                str = br.readLine();
            }
            
            // Create 2D representation of obstacles in text form
            grid = new char[strs.size()][strs.get(0).length()];
            for (int i = 0; i < strs.size(); i++)
                grid[i] = strs.get(i).toCharArray();
                
            br.close();
        }
        catch (IOException ex)
        {}
    }
    
    /*
     * Update viewing screen
     */
    public void paint()
    {
        // Canvas to draw on
        GreenfootImage pic = new GreenfootImage(600, 400);
        
        if (mainScreen)
        {
            
        }
        else
        {
            // Draw a scrolling background
            GreenfootImage back = stonePic;
            for (int i = -(myShip.getPos().getY() % back.getHeight()); i < 400; i += back.getHeight())
                for (int k = -(myShip.getPos().getX() % back.getWidth()); k < 600; k += back.getWidth())
                    pic.drawImage(stonePic, k, i);
            
            // Draw the blocks (walls) on screen relative to ship/centered location
            for (Point p : blocks)
            {
                int x = getXSet(p.getX());
                int y = getYSet(p.getY());
                if (x + 30 > 0 && x < 600 && y + 30 > 0 && y < 400)
                    pic.drawImage(blockPic, x, y);
            }
            
            // Draw trails of ship, then 
            for (Trail t : myShip.getTrail())
                pic.drawImage(t.getPic(), getXSet(t.getPos().getX()) - 40, getYSet(t.getPos().getY()) - 40);
            
            // Draw Shades
            for (Shade s : shades)
            {
                int x = getXSet(s.getX());
                int y = getYSet(s.getY());
                if (x + Shade.getWidth() / 2 > 0 && x - Shade.getWidth() / 2 < 600 && y + Shade.getHeight() / 2 > 0 && y - Shade.getHeight() / 2 < 400)
                    pic.drawImage(s.getPic(), x - Shade.getWidth() / 2, y - Shade.getHeight() / 2);
            }
            
            // Draw Missles
            GreenfootImage mPic = new GreenfootImage(3, 3);
            mPic.setColor(Color.WHITE);
            mPic.drawOval(0, 0, 2, 2);
            for (Missle m : missles)
                pic.drawImage(mPic, getXSet(m.getX()) - 1, getYSet(m.getY()) - 1);
            
            // Draw Bursts
            for (Burst b : bursts)
                pic.drawImage(b.getPic(), getXSet(b.getPos().getX()) - b.getWidth() / 2, getYSet(b.getPos().getY()) - b.getHeight() / 2);
            
            // Draw actual ship center screen
            pic.drawImage(myShip.getPic(), 300 - 40, 200 - 40);
            
            // Create an image to overlay the viewing screen to act as a shadow
            GreenfootImage shadow = new GreenfootImage(600, 400);
            Graphics2D g = (Graphics2D)shadow.getAwtImage().getGraphics();
            g.setColor(Color.BLACK);
            g.fillRect(0, 0, 600, 400);
            g.setColor(new Color(0, 0, 0, 0));
//             g.setPaintMode();
            // Draw ovals centered over all lights, relative to ship/centered location
            for (Light l : lights)
            {
                pic.drawImage(l.getPic(), getXSet(l.getX() - Light.getWidth() / 2), getYSet(l.getY() - Light.getHeight() / 2));
                g.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_IN,(float)1));
                GreenfootImage fog2 = new GreenfootImage(fogPic);
                fog2.scale(l.getExpand(), l.getExpand());
                g.drawImage(fog2.getAwtImage(), null, getXSet(l.getX() - l.getExpand() / 2), getYSet(l.getY() - l.getExpand() / 2));
                //g.fillOval(getXSet(l.getX() - l.getExpand() / 2), getYSet(l.getY() - l.getExpand() / 2), l.getExpand(), l.getExpand());
            }
            // Reverse to make shadow effect, making ovals drawn around lights to become clear, and everything else darkened
//             g.setXORMode(new Color(0, 0, 0, 0));
//             g.fillRect(0, 0, 600, 400);
            g.dispose();
            shadow.setTransparency((int)darkness);
            pic.drawImage(shadow, 0, 0);
            
            g = (Graphics2D)pic.getAwtImage().getGraphics();
            for (int i = 0; i < Math.ceil(myShip.getHealth()); i++)
            {
                int trans = 140;
                if (i == Math.ceil(myShip.getHealth()) - 1)
                {
                    double over = myShip.getHealth() - (int)myShip.getHealth();
                    trans = (int)(over % .2 > .1 ? (over % .2d - .1d) / .1d * 140d : (.1d - over % .2d) / .1 * 140d);
                }
                if (i < 6)
                    g.setColor(new Color(255, (int)((double)i / 6d * 255d), 0, trans));
                else
                    g.setColor(new Color(255 - (int)((double)(i - 6) / 6d * 255d), 255, 0, trans));
                g.fillRoundRect(300 - centerHealth / 2 + i * 12, 378, 10, 20, 5, 5);
            }
            
            g.dispose();
            
            pic.setColor(new Color(255, 255, 255, 140));
            pic.setFont(scoreFont);
            pic.drawString("" + score, 300 - centerScore / 2, 28);
            
            String display = "";
            
            if (myShip.getHealth() < 1)
            {
                display = "GAME OVER";
                if (getWorld().getObjects(ScoreBoard.class).isEmpty())
                {
                    if (UserInfo.isStorageAvailable())
                    {
                        UserInfo ui = UserInfo.getMyInfo();
                        int oldScore = ui.getScore();
                        if (score > oldScore)
                        {
                            ui.setScore(score);
                            ui.store();
                        }
                        if (score >= 1000 || oldScore >= 1000)
                            if (CodeGenerator.haveUser())
                            {
                                getWorld().removeObjects(getWorld().getObjects(CodeDisplay.class));
                                CodeDisplay cd = new CodeDisplay(CODE_ID);
                                getWorld().addObject(cd, cd.getImage().getWidth() / 2, 6);
                            }
                        getWorld().addObject(new ScoreBoard(500, 400), getWorld().getWidth() / 2, getWorld().getHeight() / 2);
                    }
                }
            }
            else if (darkness > 200)
                display = "" + (int)((230d - darkness) / 30d * 10);
            if (display.length() != 0)
            {
                Graphics2D graphics = (Graphics2D)(new GreenfootImage(1, 1)).getAwtImage().createGraphics();
                graphics.setFont(scoreFont);
                FontMetrics fm = graphics.getFontMetrics();
                int width = fm.charsWidth((display).toCharArray(), 0, (display).length());
                graphics.dispose();
                pic.setColor(Color.RED);
                pic.drawString(display, 300 - width / 2, 180);
            }
        }
        
        // Set image for displaying
        setImage(pic);
    }
    
    /*
     * Takes an x value for some object, relates to location of ship's location centered for viewing screen, returns appropriate drawing x value for the object
     */
    private int getXSet(int x)
    {
        return 300 - (myShip.getPos().getX() - x);
    }
    
    /*
     * Takes a y value for some object, relates to location of ship's location centered for viewing screen, returns appropriate drawing y value for the object
     */
    private int getYSet(int y)
    {
        return 200 - (myShip.getPos().getY() - y);
    }
    
    /*
     * Active actions for game
     */
    public void act() 
    {
        if (mainScreen)
        {
            if (Greenfoot.isKeyDown("enter"))
            {
                if (!keysPressed.contains(" enter "))
                {
                    mainScreen = false;
                    newGame();
                    keysPressed += " enter ";
                }
            }
            else
                keysPressed = keysPressed.replace(" enter ", "");
        }
        else
        {
            if (myShip.getHealth() < 1)
            {
                if (Greenfoot.isKeyDown("enter"))
                {
                    if (!keysPressed.contains(" enter "))
                    {
                        getWorld().removeObjects(getWorld().getObjects(ScoreBoard.class));
                        mainScreen = true;
                        keysPressed += " enter ";
                    }
                }
                else
                    keysPressed = keysPressed.replace(" enter ", "");
            }
            else
            {
                // Listen for controls for ship
                if (Greenfoot.isKeyDown("a"))
                    myShip.turn(-5);
                else if (Greenfoot.isKeyDown("d"))
                    myShip.turn(5);
                if (Greenfoot.isKeyDown("w"))
                    myShip.throttle();
                else if (Greenfoot.isKeyDown("s"))
                    myShip.slow();
                
                if (Greenfoot.isKeyDown("space"))
                {
                    if (!keysPressed.contains(" space "))
                    {
                        missles.add(new Missle(myShip.getPos(), myShip.getDirection()));
                        keysPressed += " space ";
                    }
                }
                else
                    keysPressed = keysPressed.replace(" space ", "");
                    
                // Allow the ship travel
                myShip.go(this);
                
                Graphics2D graphics = (Graphics2D)(new GreenfootImage(1, 1)).getAwtImage().createGraphics();
                graphics.setFont(scoreFont);
                FontMetrics fm = graphics.getFontMetrics();
                int width = fm.charsWidth((score + "").toCharArray(), 0, (score + "").length());
                graphics.dispose();
                centerScore -= (new Integer(centerScore)).compareTo(new Integer(width));
                
                centerHealth -= (new Integer(centerHealth)).compareTo(new Integer(((int)myShip.getHealth()) * 12 + (myShip.getHealth() < 12 ? 12 : 0)));
                
                // Allow lights to travel
                for (int i = 0; i < lights.size(); i++)
                {
                    lights.get(i).go();
                    if (lights.get(i).getExpand() == 180 && Math.sqrt(Math.pow(myShip.getPos().getX() - lights.get(i).getX(), 2) + Math.pow(myShip.getPos().getY() - lights.get(i).getY(), 2)) < 35)
                        lights.get(i).startExpand();
                    else if (lights.get(i).getExpand() > 740)
                    {
                        lights.remove(lights.get(i--));
                        darkness = 25;
                        if (Math.random() > .8 && lights.size() == 0)
                        {
                            addLight();
                            addLight();
                            addLight();
                        }
                        break;
                    }
                }
                
                if (lights.size() < 2 && Greenfoot.getRandomNumber(100) == 7)
                    addLight();
                
                Shade.num = Shade.nextNum;
                Shade.nextNum = 0;
                // Allow Shades to travel and if in range, damage Ship
                for (Shade s : shades)
                {
                    s.go(myShip.getPos(), this);
                    if (s.isArmed() && s.getHealth() > 0 && Math.sqrt(Math.pow(s.getX() - myShip.getPos().getX(), 2) + Math.pow(s.getY() - myShip.getPos().getY(), 2)) < 40)
                    {
                        myShip.hurt();
                        bursts.add(new Burst(new Point(s.getX() + (int)(Math.cos(s.getDirection() * Math.PI / 180) * Shade.getWidth() / 2), s.getY() + (int)(Math.sin(s.getDirection() * Math.PI / 180) * Shade.getHeight() / 2)), Color.RED, true));
                        s.shotMissle();
                    }
                }
                
                for (int i = 0; i < missles.size(); i++)
                {
                    missles.get(i).go();
                    if (checkWall(missles.get(i).getX(), missles.get(i).getY()))
                        missles.remove(missles.get(i--));
                    else
                        for (int k = 0; k < shades.size(); k++)
                            if (shades.get(k).getHealth() > 0 && Math.sqrt(Math.pow(missles.get(i).getX() - shades.get(k).getX(), 2) + Math.pow(missles.get(i).getY() - shades.get(k).getY(), 2)) < 30)
                            {
                                shades.get(k).damage();
                                if (shades.get(k).getHealth() < 1)
                                    score += 25 + (int)Math.sqrt(score);
                                bursts.add(new Burst(new Point(missles.get(i).getX(), missles.get(i).getY()), Color.PINK, false));
                                missles.remove(missles.get(i--));
                                break;
                            }
                }
                for (int i = 0; i < bursts.size(); i++)
                {
                    bursts.get(i).go();
                    if (bursts.get(i).isDone())
                        bursts.remove(bursts.get(i--));
                }
                
                boolean expanding = false;
                for (Light l : lights)
                    if (l.getExpand() != 180)
                        expanding = true;
                
                // Increase darkness for shadow
                if (darkness < 230)
                {
                    if (!expanding)
                        darkness += (darkness < 100 ? .4 : .04);
                }
                else
                    myShip.die();
            }
        }
        // Update screen, skipping some time
        if (skip++ == 2)
        {
            paint();
            skip = 0;
        }
        
        if (Greenfoot.isKeyDown("a"))
        {}
        if (Greenfoot.isKeyDown("d"))
        {}
        if (Greenfoot.isKeyDown("w"))
        {}
        if (Greenfoot.isKeyDown("s"))
        {}
        if (Greenfoot.isKeyDown("space"))
        {}
    }
    
    public void xorPic(GreenfootImage pic)
    {
        for (int i = 0; i < pic.getWidth(); i++)
            for (int k = 0; k < pic.getHeight(); k++)
                if (pic.getColorAt(i, k).getAlpha() == 0)
                    pic.setColorAt(i, k, Color.BLACK);
                else
                    pic.setColorAt(i, k, new Color(0, 0, 0, 0));
    }
    
    public void addLight()
    {
        Point p = new Point(Greenfoot.getRandomNumber(1250), Greenfoot.getRandomNumber(600));
        while (Math.sqrt(Math.pow(p.getX() - myShip.getPos().getX(), 2) + Math.pow(p.getY() - myShip.getPos().getY(), 2)) < 300)
            p = new Point(Greenfoot.getRandomNumber(1250), Greenfoot.getRandomNumber(600));
        lights.add(new Light(p.getX(), p.getY()));
    }
    
    /*
     * Given a map location, check for an obstacle on the grid
     */
    public boolean checkWall(double x, double y)
    {
        return grid[(int)y / 30][(int)x / 30] == '#';
    }
    
    public static void createLightPic()
    {
        GreenfootImage pic = new GreenfootImage(360, 360);
        for (int i = 0; i < 360; i++)
            for (int k = 0; k < 360; k++)
            {
                double distance = Math.sqrt(Math.pow(k - 360/2, 2) + Math.pow(i - 360/2, 2));
                //distance -= distance % 3;
        //System.out.println(distance + "");
                //if (distance > 90)
                    pic.setColorAt(k, i, new Color(0, 0, 0, distance > 180 ? 255 : (int)((distance) / 180d * 255d)));
            }
        try
        {
            ImageIO.write(pic.getAwtImage(), "png", new File("images/" + "foggy2" + ".png"));
        }
        catch (IOException ex)
        {}
    }
}