import greenfoot.*;
/**
 * Write a description of class Light here.
 * 
 * @author Taylor Born
 * @version January 2011
 */
public class Light  
{
    // Original picture for drawing this Light
    private static GreenfootImage lightPic = new GreenfootImage("light.png");
    
    // The rotation(spinning) of the image to be drawn
    private int rotation;
    
    // The current direction at which this Light is moving
    private int direction;
    
    // Spot of destination
    private Point dest = new Point(Greenfoot.getRandomNumber(1270), Greenfoot.getRandomNumber(600));
    
    // 
    private int wait;
    
    // Map location of this Light
    private double x, y;
    
    private int expand = 180;
    
    public Light(int x, int y)
    {
        this.x = x;
        this.y = y;
    }
    
    /*
     * Travel
     */
    public void go()
    {
        if (expand > 180)
            expand += Math.pow(1.2, Math.sqrt(expand) / 1.5);
        else
        {
            // Rotate
            rotation += 5;
            
            if (wait > 0)
            {
                wait--;
                return;
            }
            
            // The direction towards destination
            double dir = Math.atan2(dest.getY() - y, dest.getX() - x) * 180 / Math.PI;
            if (dir < 0)
                dir += 360;
            
            // The differences of degrees between the current direction and the direction towards destination
            //      'inCircle' counts difference not going accross the 0 and 360 ends
            double inCircle = Math.abs(dir - direction);
            //      'overCircle' does count difference going accross the 0 and 360 ends
            double overCircle = dir > direction ? (360 - dir + direction) : (360 - direction + dir);
            
            // Check which difference is smaller
            if (inCircle < overCircle)
            {
                if (inCircle > 1) // Only do something if meaningful
                {
                    // Since staying away from 0 and 360 ends, change direction closer to direction towards destination
                    if (direction > dir)
                        direction -= 2;
                    else
                        direction += 2;
                }
            }
            else
            {
                if (overCircle > 1) // Only do something if meaningful
                {
                    // Since going over the 0 or 360 end, change direction closer to the end which direction towards destination is not in the way
                    if (direction > dir)
                        direction += 2;
                    else
                        direction -= 2;
                }
            }
            
            // Keep the direction value in nice range
            if (direction >= 360)
                direction -= 360;
            else if (direction < 0)
                direction += 360;
            
            // Move location in set direction
            x += Math.cos(direction * Math.PI / 180);
            y += Math.sin(direction * Math.PI / 180);
            
            // If in range of destination, set new destination
            if (Math.sqrt(Math.pow(x - dest.getX(), 2) + Math.pow(y - dest.getY(), 2)) < 30)
            {
                dest = new Point(Greenfoot.getRandomNumber(1270), Greenfoot.getRandomNumber(600));
                if (Greenfoot.getRandomNumber(7) == 2)
                    wait = 175 + Greenfoot.getRandomNumber(250);
            }
        }
    }
    
    /*
     * Return the image rotated by the spin rotation value, for drawing
     */
    public GreenfootImage getPic()
    {
        GreenfootImage p = new GreenfootImage(lightPic);
        p.rotate(rotation);
        return p;
    }
    
    /*
     * Return the map location x value
     */
    public int getX()
    {
        return (int)x;
    }
    
    /*
     * Return the map location y value
     */
    public int getY()
    {
        return (int)y;
    }
    
    public int getExpand()
    {
        return expand;
    }
    public void startExpand()
    {
        expand = 200;
    }
    public static int getWidth()
    {
        return lightPic.getWidth();
    }
    public static int getHeight()
    {
        return lightPic.getHeight();
    }
}